Option Explicit

Class Compressor
	Private InputData
	Private OutputData
	Private FlagBuffer
	Private FlagPosition
	Private DictStep
	Private BestLength()
	Private BestData()
	Private DictCount()
	Private DictCode()
	Private DictPointer()

	'Private Sub Class_Initialize
	'End Sub

	'Private Sub Class_Terminate
	'End Sub

	Private Sub InitializeCompressor
		Dim CurrentSlot

		DictStep = 4369 '1

		Redim DictCount(65535)
		Redim DictCode(65535)
		Redim DictPointer(65535)

		For CurrentSlot = 0 To 65535
			DictCount(CurrentSlot) = 0
			DictCode(CurrentSlot) = -1
			DictPointer(CurrentSlot) = -1
		Next

		Redim BestLength(InputData.Size - 1)
		Redim BestData(InputData.Size - 1)

		FlagBuffer = 256
	End Sub

	Private Sub TerminateCompressor
		DictStep = Empty
		FlagBuffer = Empty
		FlagPosition = Empty
		Erase DictCount
		Erase DictCode
		Erase DictPointer
		Erase BestLength
		Erase BestData
	End Sub

	Private Sub UpdateDictionary(CurrentPosition)
		Dim CurrentCode, CurrentSlot

		'Remove the oldest pair from the dictionary
		If (CurrentPosition - 2048) >= 0 Then
			CurrentCode = InputData.ReadByteAt(CurrentPosition - 2048) + InputData.ReadByteAt(CurrentPosition - 2047) * 256
			CurrentSlot = CurrentCode
			DictCount(CurrentSlot) = DictCount(CurrentSlot) - 1
			'Find the slot that points to the start of the sliding window
			While DictPointer(CurrentSlot) <> (CurrentPosition - 2048)
				CurrentSlot = (CurrentSlot + DictStep) And 65535
			Wend
			DictCode(CurrentSlot) = -1
			DictPointer(CurrentSlot) = -1
		End If

		'Add the new pair to the dictionary
		CurrentCode = InputData.ReadByteAt(CurrentPosition) + InputData.ReadByteAt(CurrentPosition + 1) * 256
		CurrentSlot = CurrentCode
		DictCount(CurrentSlot) = DictCount(CurrentSlot) + 1
		'Find the next free slot
		While DictCode(CurrentSlot) > -1
			CurrentSlot = (CurrentSlot + DictStep) And 65535
		Wend
		DictCode(CurrentSlot) = CurrentCode
		DictPointer(CurrentSlot) = CurrentPosition
	End Sub

	Private Sub FindBestMatch(CurrentPosition)
		Dim CurrentCode, CurrentSlot, MatchCount, CurrentLength, PreviousPosition, MatchNotFinished

		'Assume that the best is an uncompressed character
		BestLength(CurrentPosition) = 1
		BestData(CurrentPosition) = InputData.ReadByteAt(CurrentPosition)

		CurrentCode = InputData.ReadByteAt(CurrentPosition) + InputData.ReadByteAt(CurrentPosition + 1) * 256
		CurrentSlot = CurrentCode

		MatchCount = DictCount(CurrentCode)

		While MatchCount > 0 'While there are occurrences of the pair in the dictionary
			CurrentLength = 2
			'Find the next slot with this code
			While (DictCode(CurrentSlot) <> CurrentCode) 'Or (DictPointer(CurrentSlot) = -1)
				CurrentSlot = (CurrentSlot + DictStep) And 65535
			Wend
			PreviousPosition = DictPointer(CurrentSlot)
			MatchNotFinished = True
			While MatchNotFinished
				If (CurrentPosition + CurrentLength) < InputData.Size Then 'If there's still data to be read
					If InputData.ReadByteAt(PreviousPosition + CurrentLength) = InputData.ReadByteAt(CurrentPosition + CurrentLength) Then
						CurrentLength = CurrentLength + 1
						If CurrentLength = 16 Then MatchNotFinished = False
					Else
						MatchNotFinished = False
					End If
				Else
					MatchNotFinished = False
				End If
			Wend
			If CurrentLength > BestLength(CurrentPosition) Then
				BestLength(CurrentPosition) = CurrentLength
				BestData(CurrentPosition) = PreviousPosition
			End If
			MatchCount = MatchCount - 1
			CurrentSlot = (CurrentSlot + DictStep) And 65535
		Wend
	End Sub

	Private Sub OutputFlag(FlagValue)
		If (FlagBuffer And 256) > 0 Then 'If the buffer is empty
			FlagPosition = OutputData.Position 'Remember the position
			OutputData.WriteByte 0 'Write dummy value
		End If

		FlagBuffer = (FlagBuffer * 2) Or (FlagValue And 1) 'Add the flag to the buffer

		If (FlagBuffer And 65536) > 0 Then 'If the buffer is full
			OutputData.WriteByteAt FlagPosition, FlagBuffer And 255 'Write it to the correct position
			FlagBuffer = 256 'Clear the buffer
		End If
	End Sub

	Private Sub ParseGreedy
		Dim CurrentPosition

		CurrentPosition = 0

		While CurrentPosition <= UBound(BestLength)
			If BestLength(CurrentPosition) = 1 Then
				OutputFlag 0
				OutputData.WriteByte BestData(CurrentPosition)
			Else
				OutputFlag 1
				If (BestLength(CurrentPosition) < 6) And (CurrentPosition - BestData(CurrentPosition) <= 32) Then
					OutputData.WriteByte 128 + (BestLength(CurrentPosition) - 2) * 32 + (CurrentPosition - BestData(CurrentPosition)) - 1
				Else
					BestData(CurrentPosition) = (BestLength(CurrentPosition) - 1) * 2048 + (CurrentPosition - BestData(CurrentPosition)) - 1
					OutputData.WriteByte (BestData(CurrentPosition) And 65280) \ 256
					OutputData.WriteByte BestData(CurrentPosition) And 255
				End If
			End If
			CurrentPosition = CurrentPosition + BestLength(CurrentPosition)
		Wend
	End Sub

	Private Sub ParseOptimal
		Dim CurrentPosition, CurrentBlock, BestBlock, NewLength

		CurrentPosition = UBound(BestLength)

		While CurrentPosition >= 0
			BestBlock = CurrentPosition
			For CurrentBlock = CurrentPosition - 1 To CurrentPosition - 15 Step -1
				If CurrentBlock >= 0 Then
					If CurrentBlock + BestLength(CurrentBlock) >= CurrentPosition + 1 Then BestBlock = CurrentBlock
				End If
			Next

			NewLength = CurrentPosition - BestBlock + 1

			If (NewLength = 1) And (BestLength(BestBlock) > 1) Then
				CurrentBlock = BestBlock
				While BestLength(CurrentBlock) > 1
					CurrentBlock = BestData(CurrentBlock)
				Wend
				BestData(BestBlock) = BestData(CurrentBlock)
			End If
			BestLength(BestBlock) = NewLength Or 256
			CurrentPosition = BestBlock - 1
		Wend

		CurrentPosition = 0

		While CurrentPosition <= UBound(BestLength)
			If (BestLength(CurrentPosition) And 256) > 0 Then
				BestLength(CurrentPosition) = BestLength(CurrentPosition) And 255

				If BestLength(CurrentPosition) = 1 Then
					OutputFlag 0
					OutputData.WriteByte BestData(CurrentPosition)
				Else
					OutputFlag 1
					If (BestLength(CurrentPosition) < 6) And (CurrentPosition - BestData(CurrentPosition) <= 32) Then
						OutputData.WriteByte 128 + (BestLength(CurrentPosition) - 2) * 32 + (CurrentPosition - BestData(CurrentPosition)) - 1
					Else
						BestData(CurrentPosition) = (BestLength(CurrentPosition) - 1) * 2048 + (CurrentPosition - BestData(CurrentPosition)) - 1
						OutputData.WriteByte (BestData(CurrentPosition) And 65280) \ 256
						OutputData.WriteByte BestData(CurrentPosition) And 255
					End If
				End If
			End If
			CurrentPosition = CurrentPosition + 1
		Wend
	End Sub

	Public Sub CompressFile(InputFile, OutputFile)
		Dim CurrentPosition

		Set InputData = New ByteStream
		InputData.LoadFromFile InputFile

		InitializeCompressor

		For CurrentPosition = 0 To InputData.Size - 2
			FindBestMatch CurrentPosition
			UpdateDictionary CurrentPosition
		Next

		'If the last byte does not belong to the end of a match, it can only be encoded as uncompressed
		BestLength(CurrentPosition) = 1
		BestData(CurrentPosition) = InputData.ReadByteAt(CurrentPosition)

		Set InputData = Nothing

		Set OutputData = New ByteStream

		'Generate the output
		'ParseGreedy '(5763 bytes)
		ParseOptimal '(5729 bytes) Maybe the parsing isn't optimal?

		'Mark the end of the stream
		OutputFlag 1
		OutputData.WriteByte 0

		'Output flags that may still be in the buffer
		If (FlagBuffer And 256) = 0 Then
			While (FlagBuffer And 65536) = 0
				FlagBuffer = FlagBuffer * 2
			Wend
			OutputData.WriteByteAt FlagPosition, FlagBuffer And 255
		End If

		OutputData.SaveToFile OutputFile
		Set OutputData = Nothing

		TerminateCompressor
	End Sub
End Class

Class Decompressor
	Private InputData
	Private OutputData

	Private Sub Decompress
		Dim EndOfData, FlagBuffer, MatchLength, MatchOffset

		EndOfData = False

		While Not(EndOfData)
			FlagBuffer = 256 + InputData.ReadByte
			While ((FlagBuffer And 65536) = 0) And Not(EndOfData)
				If (FlagBuffer And 128) > 0 Then 'Compressed block
					MatchLength = InputData.ReadByte
					If MatchLength = 0 Then
						EndOfData = True
					Else
						If (MatchLength And 128) > 0 Then
							MatchOffset = (MatchLength And 31) + 1
							MatchLength = (MatchLength And 96) \ 32 + 2
						Else
							MatchOffset = (MatchLength And 7) * 256 + InputData.ReadByte + 1'- 1
							MatchLength = (MatchLength And 120) \ 8 + 1
						End If
						While MatchLength > 0
							OutputData.WriteByte OutputData.ReadByteAt(OutputData.Position - MatchOffset)
							MatchLength = MatchLength - 1
						Wend
					End If
				Else 'Uncompressed byte
					OutputData.WriteByte InputData.ReadByte
				End If
				FlagBuffer = FlagBuffer * 2
			Wend
		Wend
	End Sub

	Public Sub DecompressFile(InputFile, OutputFile)
		Set InputData = New ByteStream
		Set OutputData = New ByteStream

		InputData.LoadFromFile InputFile

		Decompress

		OutputData.SaveToFile OutputFile

		Set InputData = Nothing
		Set OutputData = Nothing
	End Sub
End Class